(document:surround "/std/frame")
(document:insert "/std/functions")
(document:insert "/dialog/frame")
(document:envelop with-translation i18n "alterator-bootloader")


(define (listwidget-list-set-current-text listwidget lst txt)
    (listwidget
	current (or (list-index (lambda (x) (string=? txt x)) lst) 0)
    )
)

(define (listwidget-get-current-text listwidget)
    (if (= (listwidget current) -1)
	""
	(listwidget text)
    )
)

(define (listwidget-set-current-text listwidget txt)
    (and (not (= (listwidget current) -1))
	(listwidget row-item (listwidget current) txt)
    )
)

(define (listbox-set-current-pixmap listwidget lst text)
    (listwidget row-item
                (or (list-index (lambda (x) (string=? text x)) lst) -1)
                (cons #f "theme:radio-on"))
)

(define (listbox-set-empty-pixmap listwidget lst text)
    (listwidget row-item
                (or (list-index (lambda (x) (string=? text x)) lst) -1)
                (cons #f ""))
)

(define (chkwidget-set-current-state widget txt)
    (widget state (if (string=? txt "__true__") #t #f))
)

(define (woo-chkwidget-set-current-state woo widget option)
    (chkwidget-set-current-state widget (get-woo-option-from woo option))
)

(define (on-password-checked chk edt)
    (edt activity (chk state)))

(define (set-defaults-bool-option name value)
    (woo-write "/bootloader/defaults" name (if value "__true__" "__false__"))
)
(define (set-defaults-string-option name value)
    (woo-write "/bootloader/defaults" name value)
)
(define (set-image-bool-option name value)
    (and
	(not-empty-string? (cell-ref current_label))
	(woo-write
	    (string-append "/bootloader/images/" (cell-ref current_label))
	    name
	    (if value "__true__" "__false__")
	)
    )
)
(define (set-image-string-option name value)
    (and (not-empty-string? (cell-ref current_label))
	(woo-write
	    (string-append "/bootloader/images/" (cell-ref current_label))
	    name
	    value)
    )
)
(define (set-defaults-option-boot edit_widget)
    (set-defaults-string-option "boot" (edit_widget text))
    (set-defaults-string-option "root" (edit_widget text))
)
(define (set-default-image)
    (and (not-empty-string? (listwidget-get-current-text labels_listbox))
	(begin
	    (cell-set! current_label (listwidget-get-current-text labels_listbox))
	    (cell-set! default_default (cell-ref current_label))
	    (set-defaults-string-option "default" (cell-ref current_label))
	)
    )
)
(define (get-woo-option-from woo_read_result option_name)
    (woo-get-option (car woo_read_result) option_name)
)

(define (woo-listwidget-list-set-current-text woo widget list option)
    (listwidget-list-set-current-text widget list (get-woo-option-from woo option))
)


(define disks (make-cell (list)))
(define vgas (make-cell (list)))
(define vgastrings (make-cell (list)))
(define images (make-cell (list)))
(define imagefiles (make-cell (list)))
(define initrdfiles (make-cell (list)))
(define bioses (make-cell (list "default" "0" "1" "2" "3" "4" "5" "6" "7")))
(define default_default (make-cell ""))
(define default_boot (make-cell ""))
(define default_bios (make-cell ""))
(define default_vga (make-cell ""))
(define default_timeout (make-cell ""))
(define default_prompt (make-cell #t))
(define default_password (make-cell ""))
(define default_lba32 (make-cell #f))
(define default_compact (make-cell #f))
(define default_lock (make-cell #f))
(define default_restricted (make-cell #f))
(define current_label (make-cell ""))

(define (install-bootloader is_install)
    (let ((test (woo-read "/bootloader/test"))
	    (install_result (make-cell #f))
	    (install_check_result (make-cell ""))
	    (install_result_message (make-cell "")))

	(cell-set! install_check_result (get-woo-option-from test 'result))
	(cell-set! install_result_message (get-woo-option-from test 'output))
	(and (equal? (cell-ref install_check_result) "__true__") (cell-set! install_result #t))
	(cell-set! install_result_message (string-join (string-split (cell-ref install_result_message) #\|) "<br>"))
	(and
	    (and install_result is_install)
	    (woo-read "/bootloader/apply")
	)
	(if is_install
	    (if (cell-ref install_result)
		(document:popup-information
		    (string-append (i18n "Boot loader installed.")
			(string-append "<br>" (cell-ref install_result_message))))
		(document:popup-warning
	    	    (string-append (i18n "Boot loader NOT installed!")
			(string-append "<br>" (cell-ref install_result_message))))
	    )
	    (if (cell-ref install_result)
		(document:popup-information
		    (string-append (i18n "Test successful.")
			(string-append "<br>" (cell-ref install_result_message))))
		(document:popup-warning
	    	    (string-append (i18n "Test failed!")
			(string-append "<br>" (cell-ref install_result_message))))
	    )
	)
    )
)

(define (session-part1)
    (load-disks)
    (load-vgas)
    (load-imagefiles)
    (load-initrdfiles)
)

(define (session-part2)
    (load-defaults)
    (load-images-list)
    (labels_listbox (rows (cell-ref images)))
    ;(listwidget-list-set-current-text labels_listbox (cell-ref images) (cell-ref current_label))
    (listbox-set-current-pixmap labels_listbox (cell-ref images) (cell-ref default_default))
)

(define (new-session)
    (woo-write "/bootloader/newsession" "x" "x")
    (session-part1)
    (session-part2)
)

(define (autogenerate)
    (woo-write "/bootloader/generate" "x" "x")
    (session-part1)
    (session-part2)
)

(define (load-disks)
    (cell-set! disks 
	(sort
	    (woo-list-names "/bootloader/disks") string<?)))

(define (load-vgas)
    (cell-set! vgas
	    (woo-list-names "/bootloader/vgas"))
    (cell-set! vgastrings
	    (woo-list-names "/bootloader/vgastrings"))
)

(define (load-imagefiles)
    (cell-set! imagefiles
	    (woo-list-names "/bootloader/kernelfiles")))

(define (load-initrdfiles)
    (cell-set! initrdfiles 
	    (woo-list-names "/bootloader/ramdiskfiles")))

(define (load-defaults)
    (let ((defaults (woo-read "/bootloader/defaults")))
	(cell-set! default_default (get-woo-option-from defaults 'default))
	(cell-set! default_boot (get-woo-option-from defaults 'boot))

	(default_boot_cmb (rows (cell-ref disks)) text (cell-ref default_boot))

	(cell-set! default_bios (get-woo-option-from defaults 'bios))
	(default_bios_cmb (rows (cell-ref bioses)))
	(and (not-empty-string? (cell-ref default_bios))
	    (listwidget-list-set-current-text
		default_bios_cmb
		(cell-ref bioses)
		(substring (cell-ref default_bios) (- (string-length (cell-ref default_bios)) 1))))

	(cell-set! default_vga (get-woo-option-from defaults 'vga))
	(default_vga_cmb (rows (cell-ref vgastrings)))
	(listwidget-list-set-current-text default_vga_cmb (cell-ref vgas) (cell-ref default_vga))

	(cell-set! default_timeout (get-woo-option-from defaults 'timeout))
	(default_timeout_edt text (cell-ref default_timeout))

	(woo-chkwidget-set-current-state defaults default_prompt_chk 'prompt)
	(woo-chkwidget-set-current-state defaults default_lba32_chk 'lba32)
	(woo-chkwidget-set-current-state defaults default_compact_chk 'compact)
	(woo-chkwidget-set-current-state defaults default_lock_chk 'lock)
	(woo-chkwidget-set-current-state defaults default_restricted_chk 'restricted)

	(cell-set! default_password (get-woo-option-from defaults 'password))
	(default_password_chk state (not-empty-string? (cell-ref default_password)))
	(default_password_edt activity (not-empty-string? (cell-ref default_password)))
	(default_password_edt text (cell-ref default_password))
    )
)

(define (load-images-list)
    (cell-set! images
	    (woo-list-names "/bootloader/images"))
)

(define ldimg_image_root (make-cell ""))
(define ldimg_image_image (make-cell ""))
(define ldimg_image_initrd (make-cell ""))
(define ldimg_other_other (make-cell ""))
(define ldimg_image_more_vga (make-cell ""))
(define ldimg_image_more_password (make-cell ""))
(define ldimg_other_more_password (make-cell ""))

(define (load-image)
    (cell-set! current_label (listwidget-get-current-text labels_listbox))
    (and (not-empty-string? (cell-ref current_label))
	(let ((image (woo-read (string-append "/bootloader/images/" (cell-ref current_label)))))
	    (if (equal? (get-woo-option-from image 'is_kernel) "__true__")
		(begin
		    (image_options visibility #t)
		    (image_more_options visibility #t)
		    (other_options visibility #f)
		    (other_more_options visibility #f)

		    (cell-set! ldimg_image_image (get-woo-option-from image 'image))
		    (image_image_cmb current-text (cell-ref ldimg_image_image))

		    (image_append_edt text (get-woo-option-from image 'append))
		    (image_label_edt text (get-woo-option-from image 'label))

		    (cell-set! ldimg_image_initrd (get-woo-option-from image 'initrd))
		    (image_initrd_cmb current-text (get-woo-option-from image 'initrd))

		    (cell-set! ldimg_image_root (get-woo-option-from image 'root))
		    (image_root_cmb current-text (cell-ref ldimg_image_root))
		    (image_append_edt text (get-woo-option-from image 'append))

		    (cell-set! ldimg_image_more_vga (get-woo-option-from image 'vga))
		    (listwidget-list-set-current-text image_more_vga_cmb (cell-ref vgas) (cell-ref ldimg_image_more_vga))
		    (woo-chkwidget-set-current-state image image_more_readonly_chk 'read-only)
		    (woo-chkwidget-set-current-state image image_more_unsafe_chk 'unsafe)
		    (woo-chkwidget-set-current-state image image_more_lock_chk 'lock)
		    (woo-chkwidget-set-current-state image image_more_restricted_chk 'restricted)
		    (cell-set! ldimg_image_more_password (get-woo-option-from image 'password))
		    (image_more_password_chk state (not-empty-string? (cell-ref ldimg_image_more_password)))
		    (image_more_password_edt activity (not-empty-string? (cell-ref ldimg_image_more_password)))
		    (image_more_password_edt text (cell-ref ldimg_image_more_password))
		)
		(begin
		    (image_options visibility #f)
		    (image_more_options visibility #f)
		    (other_options visibility #t)
		    (other_more_options visibility #t)

		    (other_label_edt text (get-woo-option-from image 'label))
		    (cell-set! ldimg_other_other (get-woo-option-from image 'other))
		    (other_other_cmb current-text (cell-ref ldimg_other_other))
		    (woo-chkwidget-set-current-state image other_more_unsafe_chk 'unsafe)
		    (cell-set! ldimg_other_more_password (get-woo-option-from image 'password))
		    (other_more_password_chk state (not-empty-string? (cell-ref ldimg_other_more_password)))
		    (other_more_password_edt activity (not-empty-string? (cell-ref ldimg_other_more_password)))
		    (other_more_password_edt text (cell-ref ldimg_other_more_password))
		)
	    )
	)
    )
)

(define (image-remove)
    (and (not-empty-string? (cell-ref current_label))
	(begin
	    (woo-delete (string-append "/bootloader/images/" (cell-ref current_label)))
	    (load-images-list)
	    (labels_listbox (rows (cell-ref images)))
	    (load-image)
	)
    )
)

(define (new-kernel)
    (let ((result (document:popup "/bootloader/new-kernel")))
	(and (not-empty-string? result)
	    (woo-new
		(string-append "/bootloader/images/" result)
		"type" "kernel"
		"label" result)
	    (session-part2)
	)
    )
)

(define (new-other)
    (let ((result (document:popup "/bootloader/new-other")))
	(and (not-empty-string? result)
	    (woo-new
		(string-append "/bootloader/images/" result)
		"type" "other"
		"label" result)
	    (session-part2)
	)
    )
)

;;;;;;;;;;;;;;;; start ;;;;;;;;;;;;;;;;;;;

(session-part1)

(document:id main_tabbox (tabbox
    (document:id main_tab_general (tab (i18n "General Settings")
	(box orientation "horizontal"
	    (label (i18n "Install boot record to drive/partition:"))
	    (document:id default_boot_cmb
		(combobox alterability #t
		    (when selected
			(set-defaults-option-boot default_boot_cmb))))
	)
	(box orientation "horizontal"
	    (label (i18n "BIOS disk number:"))
	    (document:id default_bios_cmb
		(combobox
		    (when selected 
			(cell-set! default_bios (default_bios_cmb current-text))
			(if
			    (equal? "default" (cell-ref default_bios))
			    (cell-set! default_bios "")
			    (cell-set! default_bios (string-append "0x8"  (cell-ref default_bios))))
			(set-defaults-string-option "disk" "")
			(set-defaults-string-option "bios" "")
			(set-defaults-string-option "disk" (cell-ref default_boot))
			(set-defaults-string-option "bios" (cell-ref default_bios))
		    )))
	)
	(box orientation "horizontal"
	    (document:id default_timeout_lbl (label (i18n "Boot the default kernel/OS after (sec/10):")))
	    (document:id default_timeout_edt (edit ""
		(when changed (set-defaults-string-option "timeout" (default_timeout_edt text)))))
	)
	(box orientation "horizontal"
	    (document:id default_vga_lbl (label (i18n "Default graphics mode on text console:")))
	    (document:id default_vga_cmb (combobox
		(when selected (set-defaults-string-option "vga" (default_vga_cmb current-text)))))
	)
	(document:id default_prompt_chk
	    (checkbox (i18n "Enter LILO prompt automatically")
		(when toggled (set-defaults-bool-option "prompt" (default_prompt_chk state)))))
	(box orientation "horizontal"
	    (document:id default_lba32_chk
		(checkbox (i18n "32-bit Logical Block Addresses")
		    (when toggled (set-defaults-bool-option "lba32" (default_lba32_chk state)))))
	    (document:id default_compact_chk
		(checkbox (i18n "Use compact mode")
		    (when toggled (set-defaults-bool-option "compact" (default_compact_chk state)))))
	)
	(box orientation "horizontal"
	    (document:id default_lock_chk
		(checkbox (i18n "Record boot command lines for defaults")
		    (when toggled (set-defaults-bool-option "lock" (default_lock_chk state)))))
	    (document:id default_restricted_chk
		(checkbox (i18n "Restrict parameters")
		    (when toggled (set-defaults-bool-option "restricted" (default_restricted_chk state)))))
	)
	(box orientation "horizontal"
	    (document:id default_password_chk
		(checkbox (i18n "Require password:")
		    (when toggled
			(begin
			    (on-password-checked default_password_chk default_password_edt)
			    (if (default_password_chk state)
				(set-defaults-string-option "password" (default_password_edt text))
				(set-defaults-string-option "password" "")
			    )
			)
		    )))
	    (document:id default_password_edt
		(edit ""
		    (when changed (set-defaults-string-option "password" (default_password_edt text)))))
	)
    ))
    (document:id main_tab_oses (tab (i18n "Operating Systems")
	(box orientation "horizontal"
	    (box
		(document:id
		    labels_listbox (listbox
			(when selected (load-image))))
			(document:id add_kernel_btn (button (i18n "Add kernel")
			    (when clicked (new-kernel))))
			(document:id add_other_btn (button (i18n "Add other OS")
			    (when clicked (new-other))))
			(document:id remove_btn (button (i18n "Remove entry")
			    (when clicked (image-remove))))
			(document:id set_default_btn (button (i18n "Set default")
			    (when clicked
				(let ((old_default (cell-ref default_default)))
				    (set-default-image)
				    (listbox-set-empty-pixmap   labels_listbox (cell-ref images) old_default)
				    (listbox-set-current-pixmap labels_listbox (cell-ref images) (cell-ref default_default))))))
	    )
		(box
		    (document:id image_box_general (groupbox (i18n "General")
			(document:id image_options
			    (box visibility #f
				(box orientation "horizontal" (label (i18n "Label"))
				    (document:id image_label_edt
					(edit ""
					    (when changed
						(set-image-string-option "label" (image_label_edt text))
						(load-images-list)
						(listwidget-set-current-text labels_listbox (image_label_edt text))))))
				(box orientation "horizontal" (label (i18n "Kernel"))(separator orientation "horizontal"))
				    (box
					(document:id image_image_cmb
					    (combobox
						alterability #t
						(rows (cell-ref imagefiles))
						(when selected
						    (set-image-string-option "image" (image_image_cmb current-text))))))
				(box orientation "horizontal" (label (i18n "Initial ramdisk"))(separator orientation "horizontal"))
				    (box
					(document:id image_initrd_cmb
						(combobox
							alterability #t (rows (cell-ref initrdfiles))
					    		(when selected
								(set-image-string-option "initrd" (image_initrd_cmb current-text)))))
				)
				(box orientation "horizontal" (label (i18n "Root filesystem"))(separator orientation "horizontal"))
				    (document:id image_root_cmb (combobox alterability #t (rows (cell-ref disks))
					(when selected
					    (set-image-string-option "root" (image_root_cmb current-text)))))
				(box orientation "horizontal" (label (i18n "Extra parameters"))(separator orientation "horizontal"))
				    (document:id image_append_edt
					(edit ""
					    (when changed (set-image-string-option "append" (image_append_edt text)))))
			    )
			)
			(document:id other_options
			    (box visibility #f
				(box orientation "horizontal" (label (i18n "Label"))
				    (document:id other_label_edt
					(edit ""
					    (when changed (set-image-string-option "label" (other_label_edt text))))))
				(box orientation "horizontal" (label (i18n "Disk"))(separator orientation "horizontal"))
				(document:id other_other_cmb
				    (combobox alterability #t (rows (cell-ref disks))
					(when selected
					    (set-image-string-option "other" (other_other_cmb current-text)))))
			    )
			)
		    ))
		    (document:id image_box_more (groupbox (i18n "More")
			(document:id image_more_options
			    (box visibility #f
				(document:id image_more_vga_cmb
				    (combobox
					(rows (cell-ref vgastrings))
					(when selected (set-image-string-option "vga" (image_more_vga_cmb current-text)))))
				(document:id image_more_readonly_chk
				    (checkbox (i18n "Mount filesystem read-only")
					(when toggled (set-image-bool-option "read-only" (image_more_readonly_chk state)))))
				(document:id image_more_unsafe_chk
				    (checkbox (i18n "Do not check partition table")
					(when toggled (set-image-bool-option "unsafe" (image_more_unsafe_chk state)))))
				(document:id image_more_lock_chk
				    (checkbox (i18n "Record boot command lines for defaults")
					(when toggled (set-image-bool-option "lock" (image_more_lock_chk state)))))
				(document:id image_more_restricted_chk
				    (checkbox (i18n "Restricted parameters")
					(when toggled (set-image-bool-option "restricted" (image_more_restricted_chk state)))))
				(box orientation "horizontal"
				    (document:id image_more_password_chk
					(checkbox (i18n "Require password:")
					    (when toggled
						(begin
						    (on-password-checked image_more_password_chk image_more_password_edt)
						    (if (image_more_password_chk state)
							(set-image-string-option "password" (image_more_password_edt text))
							(set-image-string-option "password" "")
						    )
						)
					    )
					)
				    )
				    (document:id image_more_password_edt
					(edit ""
					    (when changed (set-image-string-option "password" (image_more_password_edt text)))))
				)
			    )
			)
			(document:id other_more_options
			    (box visibility #f
				(document:id other_more_unsafe_chk
				    (checkbox (i18n "Do not check partition table")
					(when toggled (set-image-bool-option "unsafe" (other_more_unsafe_chk state)))))
				(box orientation "horizontal"
				    (document:id other_more_password_chk
					(checkbox (i18n "Require password:")
					    (when toggled
						(begin
						    (on-password-checked other_more_password_chk other_more_password_edt)
						    (if (other_more_password_chk state)
							(set-image-string-option "password" (other_more_password_edt text))
							(set-image-string-option "password" "")
						    )
						)
					    )
					)
				    )
				    (document:id other_more_password_edt
					(edit ""
					    (when changed (set-image-string-option "password" (other_more_password_edt text)))))
				)
			    )
			)
		    )
		))
		)))))

(session-part2)

;(frame:on-leave (lambda() (and (apply_btn activity) 'need-confirm)))
(frame:on-apply (lambda() (install-bootloader #t)))

(frame:buttons-view
	(button (i18n "Quit") (when clicked (document:end)))
	(document:id regenerate_btn
	    (button (i18n "Generate") (when clicked (autogenerate))))
	(document:id reset_btn
		(button (i18n "Reset") (when clicked (new-session))))
	(document:id check_btn (button (i18n "Check")
		(when clicked (install-bootloader #f))))
;	(document:id apply_btn (button (i18n "Apply") inactive (when clicked (install-bootloader #t))))
	(document:id apply_btn (button (i18n "Apply") (when clicked (install-bootloader #t))))
)
